<?php

namespace data443\GDPR\Modules\NinjaForms;
use Codelight\GDPR\Components\Consent\ConsentManager;
use Codelight\GDPR\DataSubject\DataSubjectManager;

/**
 * todo: add filters!
 *
 * Class NinjaForms
 *
 * @package data443\GDPR\Modules\NinjaForms
 */
class NinjaForms
{
    public function __construct(DataSubjectManager $dataSubjectManager, ConsentManager $consentManager)
    {   
        $this->dataSubjectManager = $dataSubjectManager;
        $this->consentManager = $consentManager;

        add_filter('gdpr/data-subject/data', [$this, 'getExportData'], 20, 2);
        add_action('gdpr/data-subject/delete', [$this, 'deleteEntries']);
        add_action('gdpr/data-subject/anonymize', [$this, 'anonymizeEntries']);
        add_action('ninja_forms_after_submission', [$this,'gdpr_add_entry_to_db'], 10, 2);
    }

    public function getExportData(array $data, $email)
    {   
        $forms = $this->getValidForms($this->getForms(), 'export');
        if (!count($forms)) 
        {
            return $data;
        }
        $all_ninja_data=array();
        $count=0;
        $form_title = "";
        foreach ($forms as $form) 
        {
            $dataids = $this->getNinjaActionSettings($forms);
            if(isset($dataids[$form->get_id()]['gdprdatarequest']))
            {
                $entries = $this->getEntriesByEmail($form, $email);
                if (!count($entries)) { 
                    continue;
                }
                if($form_title == "" || $form_title != $form->get_setting( 'title' )){
                    $form_title = $form->get_setting( 'title' );
                    $title  = __('Form submission:', 'gdpr') . ' ' . $form_title;
                    $count=0;
                }
                foreach ($entries as $i => $entry) {
                    foreach($entry as $key => $value){
                        $data[$title][$count][$i][$key]= rtrim($value);
                    } 
                }
            }
            $count++;
        }
    return $data;
    }

    public function deleteEntries($email)
    {
        $forms = $this->getValidForms($this->getForms(), 'delete');
        if (!count($forms)) 
        {
            return;
        }
        foreach ($forms as $form) 
        {
            $dataids = $this->getNinjaActionSettings($forms);
            if(isset($dataids[$form->get_id()]['gdprdatarequest']))
            {   
                $entries = $this->getEntriesByEmail($form, $email);
                if (!count($entries)) 
                {
                    continue;
                }
                if($entries){
                    foreach ($entries as $entry) 
                    {
                        $meta_key="_field_".$dataids[$form->get_id()]['gdprdatarequest'];
                        $this->bulk_gdpr_delete($meta_key,$email);
                    }
                }
            }
        }
    }

    public function anonymizeEntries($email)
    {   
        global $wpdb;
        $forms = $this->getValidForms($this->getForms(), 'delete');
        if (!count($forms)) 
        {
            return;
        }
        foreach ($forms as $form) 
        {
            $dataids = $this->getNinjaActionSettings($forms);
            $form_id= $form->get_id();
            if(isset($dataids[$form_id]['gdprdatarequest']) && $dataids[$form_id]['gdpr-anonymize']=="1")
            {       
                $meta_key="_field_".$dataids[$form->get_id()]['gdprdatarequest'];
                $sub_sql = "SELECT id FROM `{$wpdb->prefix}posts` AS p
                    LEFT JOIN `{$wpdb->prefix}postmeta` AS m ON p.id = m.post_id
                    WHERE p.post_type = 'nf_sub' AND m.meta_key = %s
                    AND m.meta_value = %s";
                $subs = $wpdb->get_col( $wpdb->prepare( $sub_sql, $meta_key, $email ),0 );
                if($subs){
                    foreach($subs as $sub){
                        $fields = Ninja_Forms()->form($form_id)->get_fields();
                        foreach ($fields as $field_id => $field) 
                        {   
                            $field_key = "_field_".$field->get_id();
                            if($field->get_setting('personally_identifiable')!="")
                            {   
                                $this->bulk_gdpr_anonymize($sub, $form_id, $field_key);
                            } 
                        }
                    }               
                }                
            }
        }
    }
    // get all forms
    public function getForms()
    {   
        return Ninja_Forms()->form()->get_forms();
    }
    // get all form's settings
    public function getNinjaActionSettings($forms)
    {
        $parent=array();
        foreach ($forms as $form) {
        $form_id 	= $form->get_id();
        $models = Ninja_Forms()->form( $form_id )->get_actions();
        
        foreach($models as $model){
            $id = $model->get_id();
            $settings = $model->get_settings();
            
                if($settings['type']=="gdprdatarequest"){
                    $setting_field = $this->get_string_between($settings['gdpr-email'],"{field:","}");
                    global $wpdb;
                    $results = $wpdb->get_results( "SELECT `parent_id` from `{$wpdb->prefix}nf3_field_meta` where `key` = 'key' AND `value` = '".$setting_field."'");
                    $parent[$form_id]['gdprdatarequest'] = current($results)->parent_id;
                    if(isset($settings['gdpr-anonymize'])){
                        if($settings['gdpr-anonymize']=="1"){
                            $parent[$form_id]['gdpr-anonymize'] = true;
                        }
                    }
                }
            }
        }
        return $parent;
    }
    // get all valid form
    public function getValidForms($forms, $action)
    {
        $validForms = [];
        foreach ($forms as $form) {
            $form_id 	= $form->get_id();
            $models = Ninja_Forms()->form( $form_id )->get_actions();
            $gdpr_enabled = false;
            foreach($models as $model){
                $id = $model->get_id();
                $settings = $model->get_settings();
                if($settings['type']=="gdprdatarequest"){
                    $gdpr_enabled = true;
                }
            }
            // Skip if email is not set
            if (!$gdpr_enabled) { 
                continue;
            }else{
                $validForms[] = $form;
            }
        }        
        return $validForms;
    }
    
    public function get_string_between($string, $start, $end)
    {
        $string = ' ' . $string;
        $ini = strpos($string, $start);
        if ($ini == 0) return '';
        $ini += strlen($start);
        $len = strpos($string, $end, $ini) - $ini;
        return substr($string, $ini, $len);
    }

    public function getEntriesByEmail($form, $email)
    {   
        $form_id = $form->get_id();
        $all_forms_Data = Ninja_Forms()->form( $form_id )->get_subs();
        $all_count=0;       
        $gdpr_email_related_data=array();
        foreach($all_forms_Data as $data){
            $ninja_submit_date = get_the_date( "M j, Y @ H:i",$data->get_id());
            $form_data = $data->get_field_values();            
            if(in_array($email,$form_data)){
                $gdprdata=array(); 
                foreach($form_data as $key=> $value){
                    $exp_key = explode('_field_', $key);
                    if($exp_key[0] == ''){
                        global $wpdb;
                        $results = $wpdb->get_results( "SELECT `key` FROM `{$wpdb->prefix}nf3_fields` WHERE id = $exp_key[1]");
                        $gdprdata[current($results)->key]=$value;
                    }
               }        
               $gdpr_email_related_data[$all_count]=$gdprdata;  
               $gdpr_email_related_data[$all_count]['date_created']=$ninja_submit_date;
               $all_count++;     
            }             
            
        }  
        return $gdpr_email_related_data;    
    }

    public function bulk_gdpr_anonymize($submission_id, $form_id, $field_key)
    {
        global $wpdb;
        $sql = "UPDATE 
                {$wpdb->prefix}postmeta 
                SET meta_value = '[deleted]'
                WHERE post_id = %d
                AND meta_key =  %s";
        $q = $wpdb->query($wpdb->prepare($sql, $submission_id ,$field_key));
        if($wpdb->last_error !== '') :
        $wpdb->print_error();
        return false;
        endif;
        return true;        
    }

    public function bulk_gdpr_delete($meta_key,$email) 
    {
		global $wpdb;
        $post_result = 0;
        if($meta_key){
            $sub_sql = "SELECT id FROM `{$wpdb->prefix}posts` AS p
                LEFT JOIN `{$wpdb->prefix}postmeta` AS m ON p.id = m.post_id
                WHERE p.post_type = 'nf_sub' AND m.meta_key = %s
                AND m.meta_value = %s";
            $subs = $wpdb->get_col( $wpdb->prepare( $sub_sql,$meta_key,$email ),0 );
            /** delete post meta */
            if($subs){
                $delete_meta_query = "DELETE FROM `{$wpdb->prefix}postmeta` WHERE post_id IN ( [IN] )";
                $delete_meta_query = $this->gdpr_prepare_in( $delete_meta_query, $subs );
                $meta_result = $wpdb->query( $delete_meta_query );
                
                if ( $meta_result > 0 ) 
                {   /** delete post */
                    $delete_post_query = "DELETE FROM `{$wpdb->prefix}posts` WHERE id IN ( [IN] )";
                    $delete_post_query = $this->gdpr_prepare_in( $delete_post_query, $subs );
                    $post_result = $wpdb->query( $delete_post_query );
                }
            }
        }
	}

    protected function gdpr_prepare_in( $sql, $vals ) 
    {
		global $wpdb;
		$not_in_count = substr_count( $sql, '[IN]' );
        if ( $not_in_count > 0 ) 
        {
			$args = array( str_replace( '[IN]', implode( ', ', array_fill( 0, count( $vals ), '%d' ) ), str_replace( '%', '%%', $sql ) ) );
            for ( $i=0; $i < substr_count( $sql, '[IN]' ); $i++ ) 
            {
				$args = array_merge( $args, $vals );
			}
			$sql = call_user_func_array( array( $wpdb, 'prepare' ), array_merge( $args ) );
        }
		return $sql;
    }
    
    public function gdpr_add_entry_to_db($form_data)
    {   
        $all_keys_forable = $form_data['settings']['formContentData'];
        $consents = $this->findConsents($all_keys_forable);
        if (!count($consents)) 
        {
            return;
        }
        $models = Ninja_Forms()->form( $form_data['form_id'] )->get_actions();
        foreach($models as $model)
        {
            $id = $model->get_id();
            $settings = $model->get_settings();
            if($settings['type']=="gdprdatarequest")
            {
                $setting_field = $this->get_string_between($settings['gdpr-email'],"{field:","}");
            }
        }
        if($form_data[ 'fields' ])
        {
            foreach( $form_data[ 'fields' ] as $field ) 
            {
                if($field[ 'key' ] == $setting_field)
                {
                    $email=$field[ 'value' ];
                }
            }
        }
        if (!$email) {
            return;
        }
        $dataSubject = $this->dataSubjectManager->getByEmail($email);
        foreach ($consents as $consent) 
        {
            $dataSubject->giveConsent($consent);
        }
    }
    
    public function findConsents($entry)
    {
        $consents = [];

        foreach ($entry as $tag) 
        {
            /* @var $tag \WPCF7_FormTag */
            if ( $tag && $this->consentManager->isRegisteredConsent($tag)) 
            {
                $consents[] = $tag;
            }
        }
        return $consents;
    }
}